SignedRank

Struct SignedRank 

pub struct SignedRank { /* private fields */ }
Distribution of the Wilcoxon Signed Rank Statistic

Description

Density, distribution function, quantile function and random generation for the distribution of the Wilcoxon Signed Rank statistic obtained from a sample with size n.

Arguments

  • n: number(s) of observations in the sample(s). A positive integer, or a vector of such integers.

Details

This distribution is obtained as follows. Let x be a sample of size n from a continuous distribution symmetric about the origin. Then the Wilcoxon signed rank statistic is the sum of the ranks of the absolute values $ x[i] $ for which $ x[i] $ is positive. This statistic takes values between $ 0 $ and $ n(n+1)/2 $, and its mean and variance are $ n(n+1)/4 $ and $ n(n+1)(2n+1)/24 $, respectively.

If either of the first two arguments is a vector, the recycling rule is used to do the calculations for all combinations of the two up to the length of the longer vector.

Value

dsignrank gives the density, psignrank gives the distribution function, qsignrank gives the quantile function, and rsignrank generates random deviates.

The length of the result is determined by nn for rsignrank, and is the maximum of the lengths of the numerical arguments for the other functions.

The numerical arguments other than nn are recycled to the length of the result. Only the first elements of the logical arguments are used.

Density Plot

let sgnrank = SignedRankBuilder::new().build();
let x = <[f64]>::sequence_by(-1.0, 2.0, 0.001);
let y = x
    .iter()
    .map(|x| sgnrank.density(x).unwrap())
    .collect::<Vec<_>>();

let root = SVGBackend::new("density.svg", (1024, 768)).into_drawing_area();
Plot::new()
    .with_options(PlotOptions {
        x_axis_label: "x".to_string(),
        y_axis_label: "density".to_string(),
        ..Default::default()
    })
    .with_plottable(Line {
        x,
        y,
        color: BLACK,
        ..Default::default()
    })
    .plot(&root)
    .unwrap();

Author(s)

Kurt Hornik; efficiency improvement by Ivo Ugrina.

See Also

wilcox.test to calculate the statistic from data, find p values and so on.

Distributions for standard distributions, including dwilcox for the distribution of two-sample Wilcoxon rank sum statistic.

Examples

let plot_coords = [
    [0.0, 0.5, 0.0, 0.5],
    [0.5, 1.0, 0.0, 0.5],
    [0.0, 0.5, 0.5, 1.0],
    [0.5, 1.0, 0.5, 1.0],
];
let ns = [4.0, 5.0, 10.0, 40.0];

let root = SVGBackend::new("densities.svg", (1024, 768)).into_drawing_area();
for i in 0..4 {
    let n = ns[i];
    let x = <[f64]>::sequence(0.0, n * (n + 1.0) / 2.0, 500);
    let sgnrank = SignedRankBuilder::new().with_sample_size(n).build();
    let y = x
        .iter()
        .map(|x| sgnrank.density(x).unwrap())
        .collect::<Vec<_>>();

    Plot::new()
        .with_options(PlotOptions {
            x_axis_label: "x".to_string(),
            y_axis_label: "density".to_string(),
            title: format!("n={n}"),
            plot_left: plot_coords[i][0],
            plot_right: plot_coords[i][1],
            plot_top: plot_coords[i][2],
            plot_bottom: plot_coords[i][3],
            ..Default::default()
        })
        .with_plottable(Line {
            x,
            y,
            color: BLACK,
            ..Default::default()
        })
        .plot(&root)
        .unwrap();
}